#    This software is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This software is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with This software.  If not, see <http://www.gnu.org/licenses/>.

#-----------------------------------------------------------------------------------------#

# Author: Federico Salotti
# Organization/Company: Mr_Robot_Computers_Repair 
# Country: Italy Barga(LU)
# release: 2.1 (07 Apr 2021 17:45)

use Term::ANSIColor;
use Time::HiRes;
use feature "say";
use Getopt::Long;
use utf8;

$| = 1;
@lines = ();
@elements = ();
$mode = "";

$syntax = "";
$operator = "";
$directive = "";
$function = "";
$entity = "";
$comment = "";

$state = 0;

$source = "";
$no_comment = 0;
$no_type = 0;
use constant DEFAULT_TYPE_SPEED => 0.025;
$type_speed = 0;


$_version = "autobot V 2.0 (c)\nautobot (c) is a syntax state-machine that allows you to emulate human keyboard typing recognizing and highlighting the syntax of the programming language used\nauthor: Federico Salotti\norganization: Mr_Robot_Computers_Repair\nBarga (LU), 55051";
$_help = "-s --source = set source file \n--no-comment = skip comments\n--no-type = disable human typing emulation\n--type-speed = set human typing emulation speed (DEFAULT 25)\n-v --version = get autobot current version\n-h --help = show this message";

# get file extension

GetOptions
(
	"source=s" => \$source,
	"no-comment" => \$no_comment,
	"no-type" => \$no_type,
	"type-speed=i" => \$type_speed,
	"version" => \$version,
	"help" => \$help,
);

if($type_speed <= 0)
{
	$type_speed = DEFAULT_TYPE_SPEED;
}
else
{
	$type_speed /= 1000;
}

if($version)
{
	say $_version;
	exit;
}

if($help)
{
	say $_help;
	exit;
}

$source =~ /.*\.(.*)/;
$file_type = $1;

if($file_type eq "")
{
	print color('BOLD RED');
	print "autoBOT : ERROR : No file extension detected!\n";
	print color(RESET);
	exit;
}

if(! -f $source)
{
	print color('BOLD RED');
	print "autoBOT : ERROR : File not exists\n";
	print color(RESET);
	exit;
}

if(! -f ".config/$1.conf")
{
	print color('BOLD YELLOW');
	print "autoBOT : WARNING : No syntax rule file detected for $ARGV[0]!\n";
	print "Generic syntax rules will be applicated\n";
	print color(RESET);
	$file_type = "generic";
}
		
sub _config
{
	open($READER,"<",".config/$file_type.conf");
	while(<$READER>)
	{
		chomp;
		if(/\[(.*)\]/)
		{
			$_ = <$READER>;
			chomp;

			if($1 eq "syntax") {$syntax = $_}
			if($1 eq "operator") {$operator = $_}
			if($1 eq "directive") {$directive = $_}
			if($1 eq "function") {$function = $_}
			if($1 eq "entity") {$entity = $_}
			if($1 eq "comment") {$comment = $_}
	
		}
	}
}

sub open_file
{	
	_config;

	open($READER,"<:encoding(UTF-8)",$_[0]);
	my @content = <$READER>;
	for my $line(@content)
	{
		if($no_comment) # if --no-comment option is used discard every comment line from source parsing
		{
			if($line !~ /^$comment.*/) {push(@lines,$line)}
		}
		else {push(@lines,"$line");}
	}
	
}

sub tokenizer
{
	for $element(@lines)
	{
		my @syntax = split(/($syntax)/,$element,-1);
		for my $token(@syntax)
		{
			if($no_comment) # if --no-comment option is used discard every comment part from source parsing
			{
				if($token !~ /\s*$comment.*/ ||
				   $token =~ /("|'|`)$comment(.*)?("|'|`)/)
				{push (@elements,$token)}
			}
			else {push(@elements,$token);}
		}		
	}
}

sub print_content
{

	for my $line(@elements)
	{
		print $line_counter;

		for my $token($line)
		{
			@letters = split("",$token,-1);

			if($token =~ /\s*$comment.*/)
			{
				$state = 1;
				if($token =~ /("|'|`)$comment(.*)?("|'|`)/) {$mode = "string"}
				else {$mode = "comment"}
			}

			elsif($token =~ /\s*("|'|`).*("|'|`)$/)
			{
				$state = 1;
				$mode = "string";
			}

			elsif($token =~ /^\s*($operator).*$/g)
			{
				$state = 1;
				$mode = "operator";
				
			}

			elsif($token =~ /\s*($directive).*$/g)
			{
				$state = 1;
				$mode = "directive";
			}

			elsif($token =~ /\s*($function)$/)
			{
				$state = 1;
				$mode = "function";
			}

			elsif($token =~ /\s*($entity).*/)
			{
				$state = 1;
				$mode = "variable";
			}

			else
			{
				$state = 0;
			}

			for $letter(@letters)
			{
				if($state && $mode eq "string") {print color(YELLOW);}
				elsif($state && $mode eq "variable") {print color(CYAN);}
				elsif($state && $mode eq "operator") {print color(MAGENTA);}
				elsif($state && $mode eq "directive") {print color('BOLD GREEN');}
				elsif($state && $mode eq "function") {print color('BOLD RED');}
				elsif($state && $mode eq "comment") {print color(BLUE);}
				else {print color(RESET)};

				if($mode ne "") {print $letter;}
#				print $letter.=" [stato : $state - mode : $mode]\n";

				if(!$state) {print color(RESET);}
				if(!$no_type) {Time::HiRes::sleep($type_speed)};
			}
		}
	}
}

open_file($source);
tokenizer;
print_content;


